import inspect
import logging
import os
from functools import partial

from Pyro5.errors import format_traceback, get_pyro_traceback


try: 
    from common.loghandler import TRACELEVEL
except:
    TRACELEVEL="DONT"

default_not_detailed_errors = []

# don't import config here

def format_traceback_str(*args,detailed=True):
    return (''.join([x[:500] for x in format_traceback(*args,detailed=detailed)]))


def print_formatted_traceback(detailed=True):
    logging.error(format_traceback_str(detailed=detailed))


def get_caller_info(over):
    return 'unk'
    try:
        frame = inspect.stack()[over]
        filename = frame.filename
        line_number = frame.lineno
        return os.path.basename(filename), line_number
    except:
        return "unknown",0


class SimpleExceptionContext:
    """
    A context manager for handling exceptions in a simplified manner.

    Attributes:
        err_description (str): Description of the error context.
        return_succ: Value to return on success.
        never_throw (bool): If True, suppress all exceptions.
        always_throw (bool): If True, always raise exceptions.
        debug (bool): If True, enable debug mode.
        detailed (bool): If True, provide detailed error information.
        err_to_ignore (list): List of exceptions to ignore.
        callback (callable): Function to call on exception.
        noconfig (bool): If True, do not use configuration.
        caller: Caller information.
        logf: Logger function.
        always_print (bool): If True, always print exceptions.
    """
    ISWING=None

    def __init__(self, err_description=None,return_succ=None,never_throw=False,always_throw=False,debug=False,detailed=True,err_to_ignore=[],callback=None,noconfig=False,caller=None,logf=None,always_print=False):
        self.err_description=err_description
        self.return_succ=return_succ
        self.never_throw=never_throw
        self.always_throw=always_throw
        self.debug=debug
        self.detailed=detailed
        self.err_to_ignore=err_to_ignore
        self.callback=callback
        self.nocfg=noconfig
        if caller is None:
            caller= get_caller_info(2)
        self.caller=caller
        self.logf=logf

    def checkwing(self):
        return False
        if SimpleExceptionContext.ISWING is not None:
            return SimpleExceptionContext.ISWING

        import psutil

        # Get the current process
        current_process = psutil.Process()
        # Get the parent process
        parent_process = current_process.parent()
        # Get the name of the parent process
        parent_process_name = parent_process.name()
        SimpleExceptionContext.ISWING= 'wing' in parent_process_name
        return SimpleExceptionContext.ISWING

    def __enter__(self):
        # Code to be executed when entering the context
        tostop = os.environ.get('PYCHARM_HOSTED') == '1' or self.checkwing()
        if not self.nocfg:
            try:
                from config import config
                tostop=tostop and config.Running.StopExceptionInDebug
                if config.Running.IsTest:
                    tostop = True
                self.config = config
            except:
                self.config=None
        else:
            self.config=None
            # logging.debug("error loading config in simple exception handling. Probably fine.")

        self.do_nothing= tostop and not self.never_throw and self.return_succ is None

        return self

    def __exit__(self, exc_type, exc_value, traceback):
        # Code to be executed when exiting the context
        if self.do_nothing:
            return False

        if exc_value is None:
            return False
        try:
            if self.callback is not None:
                self.callback(exc_value)
        except:
            logging.warn("error in callback {} ".format(self.callback))
        if exc_type is not None:
            # Handle any exceptions raised within the context
            # Propagate any exceptions raised within the context
            return self.on_exception(exc_type,exc_value,traceback)

    def on_exception(self, exc_type, exc_value, traceback):
        e=exc_value
        # TmpHook.GetExceptionHook().emit(e)
        if exc_type in self.err_to_ignore and not self.never_throw:
           return False  # throw
        if self.logf is None:
            logf = logging.debug if self.debug else logging.error
        else:
            logf = self.logf
        tmpst = format_traceback_str(exc_type, exc_value, traceback , detailed=self.detailed)
        strng = "# if you see this in your traceback, you should probably inspect the remote traceback as well"
        if strng in tmpst and e.__class__ not in default_not_detailed_errors:
            logf(("".join(get_pyro_traceback())))
        if self.err_description:
            logf( '%s:' % (self.caller)  + ' ' + str(self.err_description))
        if self.err_description is None:
            logf( '%s:' % (self.caller)  + ' ' + str(e))
        if self.config is not None and self.config.Running.IsTest:
            logf(format_traceback_str(exc_type,exc_value, traceback, detailed=self.detailed))  # just in case
        elif e.__class__ not in default_not_detailed_errors and self.detailed:
            logf(format_traceback_str(exc_type,exc_value, traceback, detailed=self.detailed))
        else:
            logf('%s %s ' % (e.__class__ ,str(e))  )
        if self.always_throw:
           return False #throw
        return True

def excp_handler(exc_type,handler=lambda a,b: None):
    def decorated(func):
        if hasattr(func,'errors_to_handle'):
            func.errors_to_handle+= [exc_type]
        else:
            func.errors_to_handle=[exc_type]

        def internal(*args,**kwargs):
            try:
                return func(*args,**kwargs)
            except exc_type as e:
                handler(args[0],e) #args[0] is self in this case
        return internal
    return decorated

def simple_exception_handling(err_description=None,return_succ='undef',never_throw=False,always_throw=False,debug=False,detailed=True,err_to_ignore=[],callback=None,noconfig=False,logf=None,always_print=False):
    """
    A decorator for handling exceptions in a simplified manner within a function.

    Parameters:
        err_description (str): Description of the error context.
        return_succ: Value to return on success or when no exception occurs.
        never_throw (bool): If True, suppress all exceptions.
        always_throw (bool): If True, always raise exceptions.
        debug (bool): If True, enable debug mode.
        detailed (bool): If True, provide detailed error information.
        err_to_ignore (list): List of exceptions to ignore.
        callback (callable): Function to call on exception.
        noconfig (bool): If True, do not use configuration.
        logf: Logger function.
        always_print (bool): If True, always print exceptions.

    Returns:
        The decorated function with exception handling applied.
    """
    def decorated(func):
        caller = get_caller_info(2)
        def internal(*args,**kwargs):
            nonlocal err_to_ignore,logf

            no_exception = False
            ret=None
            if hasattr(decorated,'errors_to_handle'):
                err_to_ignore+=decorated.errors_to_handle
            try:
                with SimpleExceptionContext(err_description=err_description,return_succ=return_succ,never_throw=never_throw,always_throw=always_throw,debug=debug,detailed=detailed,err_to_ignore=err_to_ignore,callback=callback,noconfig=noconfig,caller=caller,logf=logf,always_print=always_print):

                    if TRACELEVEL!="DONT": 
                        logging.log(level=TRACELEVEL, msg=f"called: {err_description} {func}")
                    ret= func(*args,**kwargs)
                    if TRACELEVEL!="DONT":
                        logging.log(level=TRACELEVEL, msg=f"after: {err_description} {func}")

                    no_exception = True
            except Exception as e:
                if hasattr(decorated, 'errors_to_handle'):
                    if e.__class__ in decorated.errors_to_handle:
                        logf = logging.debug if debug else logging.error
                        logf(err_description) # a bit of an hack
                        #excp handler can't know about description ...
                raise

            if not no_exception and return_succ!='undef':
                return return_succ
            return ret

        return internal


    return decorated
